// EE231002 Lab14. Image Processing
// 108061112, 林靖
// Date: Dec. 28, 2019

#include <stdio.h>
#include <stdlib.h>

typedef struct sPIXEL {			// a single pixel
	unsigned char r, g, b;		//   three color components
} PIXEL;

typedef struct sIMG {			// an image of PPM style
	char header[3];				//   header, either P3 or P6
	int W, H;					//   width and height of the image
	int level;					//   intensity level of each color component
	PIXEL **PX;					//   two-dimensional array for all the pixels
} IMG;

IMG *PPMin(char *inFile);
// This function opens the inFile, reads the image data and returns a pointer
//   pointing to a newly created image data structure. 

void PPMout(IMG *p1, char *outFile);
// This function writes the image pointed by p1 to the output file outFile.

IMG *PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2);
// This function processes the image pointed by p1 performing the modifications
//   and returns the new image as a result. The arguments x1, y1 are the
//   coordinates of the lower-left corner of the box which retains the color
//   image; while x2, y2 are the coordinates of the upper-right corner of the
//   box.

int main(int argc, char *argv[])
{
	IMG *p1,
		*ee,
		*nthu;

	p1 = PPMin(argv[1]);
	ee = PPMin(argv[2]);
	nthu = PPMin(argv[3]);
	PPMcvt(p1, ee, nthu, 910, 1780, 1090, 1600);
	PPMout(p1, argv[4]);

	return 0;
}

// This function opens the inFile, reads the image data and returns a pointer
//   pointing to a newly created image data structure. 
IMG *PPMin(char *inFile)
{
	int i, j;
	IMG *pic;
	FILE *fin;

	pic = (IMG *)malloc(sizeof(IMG));
	fin = fopen(inFile, "r");
	fscanf(fin, "%s", pic->header);
	fscanf(fin, "%d %d\n%d\n",
		   &pic->W, &pic->H, &pic->level);
	pic->PX = (PIXEL **)malloc(pic->W * sizeof(PIXEL *));
	for (i = 0; i < pic->W; i++) {
		pic->PX[i] = (PIXEL *)malloc(pic->H * sizeof(PIXEL));
	}
	for (j = 0; j < pic->H; j++) {
		for (i = 0; i < pic->W; i++) {
			pic->PX[i][j].r = fgetc(fin);
			pic->PX[i][j].g = fgetc(fin);
			pic->PX[i][j].b = fgetc(fin);
		}
	}
	fclose(fin);

	return pic;
}

// This function writes the image pointed by p1 to the output file outFile.
void PPMout(IMG *p1, char *outFile)
{
	int i, j;
	FILE *fout;

	fout = fopen(outFile, "w");	//open foutput file
	fprintf(fout, "%s\n", p1->header);	//write header
	fprintf(fout, "%d %d\n%d\n",
			p1->W, p1->H, p1->level);	//write width &height
	for (j = 0; j < p1->H; j++) {
		for (i = 0; i < p1->W; i++) {
			fprintf(fout, "%c", p1->PX[i][j].r);
			fprintf(fout, "%c", p1->PX[i][j].g);
			fprintf(fout, "%c", p1->PX[i][j].b);
		}
	}
	fclose(fout);

	return;
}

// This function processes the image pointed by p1 performing the modifications
//   and returns the new image as a result. The arguments x1, y1 are the
//   coordinates of the lower-left corner of the box which retains the color
//   image; while x2, y2 are the coordinates of the upper-right corner of the
//   box.
IMG *PPMcvt(IMG *p1, IMG *ee, IMG *nthu, int x1, int y1, int x2, int y2)
{
	int i, j,
		gray,
		X, Y;

	for (j = 0; j < p1->H; j++) {
		for (i = 0; i < p1->W; i++) {
			if (i < x1 ||
				i > x2 ||
				j < y2 ||
				j > y1) {
				gray = p1->PX[i][j].r * 0.2126 +
					   p1->PX[i][j].g * 0.7152 +
					   p1->PX[i][j].b * 0.0722;
				p1->PX[i][j].r = gray;
				p1->PX[i][j].g = gray;
				p1->PX[i][j].b = gray;
			}
		}
	}

	X = (p1->W) - (ee->W);	//find location for EE logo
	Y = (p1->H) - (ee->H);	//...
	for (j = 0; j < ee->H; j++) {
		for (i = 0; i < ee->W; i++) {
			if (ee->PX[i][j].r != 255 ||
				ee->PX[i][j].g != 255 ||
				ee->PX[i][j].b != 255) {
				p1->PX[X + i][Y + j] = ee->PX[i][j];
			}
		}
	}

	X = (p1->W - nthu->W) / 2;	//find location for NTHU logo
	Y = (p1->H - nthu->H) / 2;	//...
	for (j = 0; j < nthu->H; j++) {
		for (i = 0; i < nthu->W; i++) {
			if (nthu->PX[i][j].r != 255 ||
				nthu->PX[i][j].g != 255 ||
				nthu->PX[i][j].b != 255) {
				p1->PX[X + i][Y + j].r = 255;
				p1->PX[X + i][Y + j].b = 255;
			}
		}
	}

	for (j = (y2 - 1); j <= (y1 + 1); j++) {
		for (i = (x1 - 1); i <= (x2 + 1); i++) {
			if (i <= (x1 + 1) ||
				i >= (x2 - 1) ||
				j <= (y2 + 1) ||
				j >= (y1 - 1)) {
				p1->PX[i][j].r = 0;
				p1->PX[i][j].g = 255;
				p1->PX[i][j].b = 255;
			}
		}
	}

	return p1;
}
